#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gx.h>
#include <geometrie.h>

#define Tmax 240
#define Vmax 255
#define Smax 240
#define Lmax 240

/* Types */
typedef struct part {
    GXpoint position;
    GXvector vitesse;
    double rayon;
    uchar col[3];
    double rappel;
    struct part *prec, *suiv;
} Particule;

typedef struct serp {
    Particule *pilote;
    Particule *tete;
    struct serp *prec, *suiv;
} Serpent;

typedef struct segm {
    GXpoint A,B;
    GXvector N;
    double d;
    struct segm *suiv;
} Segment;

typedef struct circ {
    GXpoint C;
    double rayon;
    struct circ *suiv;
} Cercle;


/* Prototypes */

// memoire.c
int ajouter_cercle(double x, double y, double r);
void liberer_cercles(void);
int ajouter_segment(double xa, double ya, double xb, double yb);
void liberer_segments(void);
int ajouter_particule(Serpent * serpent, GXpoint pos, GXvector vit, double rap,
	uchar *col);
void detruire_particule(Particule * particule);
int ajouter_serpent(GXpoint pos, GXvector vit, double rap, int npart, 
	uchar *col);
void liberer_serpents(void);
void detruire_serpent(Serpent * serpent);

// affichage.c
void dessine_cercles(void);
void dessine_segments(void);
void dessine_particules(Particule *p);
void dessine_serpents(void);
void decor(void);
void impact(GXpoint I);

// mouvements.c
bool intersection_part_seg(Particule * p, Segment * s, bool imp);
bool intersection_part_cer(Particule * p, Cercle * c, bool imp);
bool collision_cercles(Particule * p, bool imp);
bool collision_segments(Particule * p, bool imp);
void poursuite(Particule *b, Particule *a);
void avance_particule(Particule * p, Particule *cible);
void mouvoir_serpent(Serpent * tmp, bool chasseur);
void mouvements(void);

// etc.c
void rvb_tsl(uchar rvb[3], uchar tsl[3]);
void tsl_rvb(uchar tsl[3], uchar rvb[3]);
unsigned int aleatoire(unsigned int a, unsigned int b);

// rencontres.c
void evite(Particule * a, Particule * b);
void evitement(Serpent *s1, Serpent *s2);
bool destruction(Particule *a, Particule *tete);
void destructions(Serpent *s);
void rencontres(void);
