#include "../include/serpent.h"

/* Passage de l'espace de couleur
 * [rouge,vert,bleu]  [teinte,saturation,luminosit]
 * d'aprs http://fr.wikipedia.org/wiki/Codage_informatique_des_couleurs
 */
void rvb_tsl(uchar rvb[3], uchar tsl[3])
{
    float M, m, delta, t0;
    
    // M = max(R,V,B)
    M = ( rvb[0] > rvb[1] ) ? rvb[0] : rvb[1] ;
    M = ( M > rvb[2] ) ? M : rvb[2] ;

    // m = min(R,V,B)
    m = ( rvb[0] < rvb[1] ) ? rvb[0] : rvb[1] ;
    m = ( m < rvb[2] ) ? m : rvb[2] ;

    // delta = M - m ( l'amplitude )
    delta = M - m;

    // Si R est suprieur  V et B
    if ( rvb[0] > rvb[1] && rvb[0] > rvb[2] )
	t0 = (float)( rvb[1] - rvb[2] ) / delta;
    // Si V est suprieur  R et B
    else if ( rvb[1] > rvb[0] && rvb[1] > rvb[2] )
	t0 = (float)( rvb[2] - rvb[0] ) / delta + 2.;
    // Si B est suprieur  R et V
    else
	t0 = (float)( rvb[0] - rvb[1] ) / delta + 4.;

    // Calcul de la teinte
    tsl[0] = (uchar)( t0 * Tmax / 6 ) % Tmax;

    // Calcul de la saturation
    if ( M + m <= Vmax )
	tsl[1] = (uchar)(delta / ( M + m ) * Smax);
    else
	tsl[1] = (uchar)(delta / ( 2 * Vmax - ( M + m ) ) * Smax);

    // Calcul de la luminosit
    tsl[2] = (uchar)((M + m) / (2 * Vmax) * Lmax);
}

/* Passage de l'espace de couleur
 * [teinte,saturation,luminosit]  [rouge,vert,bleu]
 * d'aprs http://fr.wikipedia.org/wiki/Codage_informatique_des_couleurs
 */
void tsl_rvb(uchar tsl[3], uchar rvb[3])
{
    float t0, M, m, delta;

    t0 = 6 * (float) tsl[0] / Tmax;

    if ( tsl[2] <= Lmax / 2 ) {
	M = Vmax * tsl[2] / Lmax * ( 1 + tsl[1] / Smax );
	m = Vmax * tsl[2] / Lmax * ( 1 - tsl[1] / Smax );
    } else {
	M = Vmax * ( tsl[1] / Lmax * ( 1 - tsl[1] / Smax ) + tsl[1] / Smax );
	M = Vmax * ( tsl[1] / Lmax * ( 1 + tsl[1] / Smax ) - tsl[1] / Smax );
    }

    delta = M - m;

    // Calcul des couleurs
    switch((int)floor(t0)) {
	case 0:
	    rvb[2] = m;
	    rvb[1] = m + t0 * delta;
	    rvb[0] = M;
	    break;
	case 1:
	    rvb[2] = m;
	    rvb[0] = m + ( 2 - t0 ) * delta;
	    rvb[1] = M;
	    break;
	case 2:
	    rvb[0] = m;
	    rvb[2] = m + ( t0 - 2 ) * delta;
	    rvb[1] = M;
	    break;
	case 3:
	    rvb[0] = m;
	    rvb[1] = m + ( 4 - t0 ) * delta;
	    rvb[2] = M;
	    break;
	case 4:
	    rvb[1] = m;
	    rvb[0] = m + ( t0 - 4 ) * delta;
	    rvb[2] = M;
	    break;
	case 5:
	    rvb[1] = m;
	    rvb[2] = m + ( 6 - t0 ) * delta;
	    rvb[0] = M;
	    break;
    }
}

/* Retourne un entier alatoire entre a et b en utilisant le gnrateur alatoire
 * de donnes de Linux, /dev/urandom
 */
unsigned int aleatoire(unsigned int a, unsigned int b)
{
    FILE *urandom;
    unsigned int r;

    /* On change a et b si b est plus petit que a */
    if ( b < a ) {
	r = b;
	b = a;
	a = r;
    }

    if ( (urandom = fopen("/dev/urandom","r")) == NULL ) {
	fprintf(stderr,"Erreur : impossible de gnrer un nombre alatoire");
	return b;
    }

    /* Lecture d'un entier dans /dev/urandom */
    fread(&r,sizeof(int),1,urandom);
    fclose(urandom);

    /* Ajustement de l'entier pour tre dans les bornes spcifies */
    r = r%(b-a+1) + a;

    return r;
}


