#include <geometrie.h>

/* Retourne le vecteur entre A et B */
GXvector Vector(GXpoint A, GXpoint B)
{
    GXvector V;
    V.x = B.x - A.x;
    V.y = B.y - A.y;
    return V;
}

/* Retourne le vecteur de coordonnes (x,y) */
GXvector Vector_XY(double x, double y)
{
    GXvector A;
    A.x = x;
    A.y = y;
    return A;
}

/* Retourne le produit scalaire de deux vecteurs */
double ProdScal(GXvector A, GXvector B)
{
    return A.x*B.x + A.y*B.y;
}

/* Retourne le rel associ  la 3e composante (Z) du vecteur rsultat du 
 * produit vectoriel : u /\ v
 */
double ProdVect(GXvector U, GXvector V)
{
    return U.x*V.y - V.x*U.y;
}

/* Retourne 1 si l'orientation du triangle ABC est positive, 0 sinon. */
int OrientationT(GXpoint A, GXpoint B, GXpoint C)
{
    // Produit vectoriel
    float z = (B.x-A.x)*(C.y-A.y) - (B.y-A.y)*(C.x-A.x);
    
    if ( IsZero(z) )
	return 0;
    else if ( z > 0 )
	return 1;
    else 
	return -1;
}

/* Retourne le carr de la norme de U */
double SqrNorm(GXvector U)
{
    return SQR(U.x) + SQR(U.y);
}

/* Retourne l'angle en radians entre AB et AC */
double AngleRadian(GXpoint A, GXpoint B, GXpoint C)
{
    GXvector AB = Vector(A,B), AC = Vector(A,C);
    double p = ProdScal(AB,AC);
    double cos = p / sqrt(SqrNorm(AB)*SqrNorm(AC));
    return OrientationT(A,B,C) * acos(cos);
}

/* Retourne 1 s'il y a intersection entre [AB] et [CD] et met la position de
 * l'intersection dans P
 */
bool InterSegment(GXpoint A, GXpoint B, GXpoint C, GXpoint D, GXpoint *P)
{
    double scal, t;
    GXvector N, CA;

    // Normale  [AB]
    N.x = A.y - B.y;
    N.y = B.x - A.x;

    scal = ProdScal(N,Vector(C,D));
    if ( IsZero(scal) ) // Pas d'intersection
	return FALSE;

    CA = Vector(C,A);
    t = ProdScal(N,CA)/scal;
    if ( t < 0.0 || t > 1.0 ) // L'intersection n'est pas sur [CD]
	return FALSE;

    // Normale  [CD]
    N.x = C.y - D.y;
    N.y = D.x - C.x;

    t = ProdScal(N,CA)/scal;
    if ( t < 0.0 || t > 1.0 ) // L'intersection n'est pas sur [AB]
	return FALSE;

    if ( P != NULL ) {
	*P = A;
	P->x += t*(B.x-A.x);
	P->y += t*(B.y-A.y);
    }
    
    return TRUE;
}

/* Calcul le projet orthogonal de C sur [AB]. Stock dans P */
double ProjPntDte(GXpoint A, GXpoint B, GXpoint C, GXpoint *P)
{
    double t, scal, norme;

    norme = SQR(B.x-A.x) + SQR(B.y-A.y); // SQR(x) = x*x

    if ( IsZero(norme) ) // Cas A = B
    {
	*P = A;
	return 0.0;
    }

    scal = (B.x-A.x)*(C.x-A.x) + (B.y-A.y)*(C.y-A.y);
    t = 1.0 * ( scal / norme );

    P->x = (1.0-t)*A.x + t*B.x;
    P->y = (1.0-t)*A.y + t*B.y;
    
    return t;
}

