#include "../include/serpent.h"

extern Segment * segments;
extern Cercle * cercles;
extern Serpent * serpents;
extern Serpent * cible;
extern uchar rouge[3], vert[3], bleu[3];

int ajouter_cercle(double x, double y, double r)
{
    Cercle * tmp;
    
    if ( (tmp = malloc(sizeof(Cercle))) == NULL )
	return 0;
    
    tmp->C = gx_Point(x,y);
    tmp->rayon = r;
    tmp->suiv = cercles;
    cercles = tmp;

    return 1;
}

void liberer_cercles(void)
{
    Cercle *tmp;

    while ( cercles != NULL ) {
	tmp = cercles->suiv;
	free(cercles);
	cercles = tmp;
    }
}

int ajouter_segment(double xa, double ya, double xb, double yb)
{
    Segment * tmp;

    if ( (tmp = malloc(sizeof(Segment))) == NULL )
	return 0;

    tmp->A = gx_Point(xa,ya);
    tmp->B = gx_Point(xb,yb);

    // Normale au vecteur AB
    tmp->N.x = -(yb - ya);
    tmp->N.y = xb - xa;

    tmp->d = SqrNorm(tmp->N);

    // Chainage
    tmp->suiv = segments;
    segments = tmp;

    return 1;
}

void liberer_segments(void)
{
    Segment * tmp;

    while ( segments != NULL ) {
	tmp = segments->suiv;
	free(segments);
	segments = tmp;
    }
}

int ajouter_particule(Serpent * serpent, GXpoint pos, GXvector vit, double rap,
	uchar *col)
{
    Particule *part;

    if ( (part = malloc(sizeof(Particule))) == NULL )
	return 0;

    part->position = pos;
    part->vitesse = vit;
    part->rappel = rap;
    part->rayon = 1.;
    if ( col != NULL )
	memcpy(part->col,col,3);
    part->prec = part->suiv = NULL;

    // Chainage
    part->suiv = serpent->pilote;
    if ( serpent->pilote != NULL )
	serpent->pilote->prec = part;
    serpent->pilote = part;
    serpent->tete = serpent->pilote->suiv;
    
    return 1;
}

void detruire_particule(Particule * particule)
{
    if ( particule->prec != NULL )
	particule->prec->suiv = particule->suiv;
    if ( particule->suiv != NULL )
	particule->suiv->prec = particule->prec;
    free(particule);
}

void liberer_particules(Particule ** particule)
{
    Particule *tmp;
    
    while ( *particule != NULL ) {
	tmp = (*particule)->suiv;
	detruire_particule(*particule);
	*particule = tmp;
    }
}

int ajouter_serpent(GXpoint pos, GXvector vit, double rap, int npart,
	uchar *col)
{
    uchar tsl[3];
    uchar couleur[3];
    Serpent * tmp;

    // Cration de couleur si elle n'est pas prcise, diffrente du rouge
    // car utilise pour les chasseurs
    if ( col == NULL ) {
	tsl[0] = aleatoire(20,220);
	tsl[1] = 240;
	tsl[2] = 120;
	tsl_rvb(tsl,couleur);
    } else
	memcpy(couleur,col,3);

    if ( npart < 1 ) {
	printf("Pas assez de particules !");
	return 0;
    }

    if ( (tmp = malloc(sizeof(Serpent))) == NULL )
	return 0;

    tmp->pilote = tmp->tete = NULL;

    while ( npart > 0 ) {
	ajouter_particule(tmp,pos,vit,rap,couleur); // particules visibles
	// On dcale les particules pour viter qu'elles se superposent
	pos.x += vit.x;
	pos.y += vit.y;
	// Dcalage de couleur pour la cible
	if ( cible == NULL ) {
	    rvb_tsl(couleur,tsl);
	    tsl[0] += 5;
	    if ( tsl[0] > 240 )
		tsl[0] -= 240;
	    tsl_rvb(tsl,couleur);
	}
	npart--;
    }
    ajouter_particule(tmp,pos,vit,rap/2,NULL); // pilote

    // Chainage
    tmp->prec = NULL;
    if ( serpents != NULL )
	serpents->prec = tmp;
    tmp->suiv = serpents;
    serpents = tmp;

    return 1;
}

void detruire_serpent(Serpent * serpent)
{
    if ( serpent->pilote != NULL )
	liberer_particules(&(serpent->pilote));

    if ( serpent == serpents ) { // Premier serpent de la liste
	serpents = serpents->suiv;
	if ( serpents != NULL )
	    serpents->prec = NULL;
    } else { // Autre
	if ( serpent->prec != NULL )
	    serpent->prec->suiv = serpent->suiv;
	if ( serpent->suiv != NULL )
	    serpent->suiv->prec = serpent->prec;
    }

    free(serpent);
}

void liberer_serpents(void)
{
    Serpent * tmp;

    while ( serpents != NULL ) {
	tmp = serpents->suiv;
	liberer_particules(&(serpents->pilote));
	free(serpents);
	serpents = tmp;
    }

    cible = NULL;
}

