#include "../include/serpent.h"

extern Segment * segments;
extern Cercle * cercles;
extern Serpent * serpents;
extern Serpent * cible;

/* Cherche une intersection entre le vecteur vitesse de p et s et affiche un
 * impact s'il y a et si imp = VRAI
 */
bool intersection_part_seg(Particule * p, Segment * s, bool imp)
{
    double scal, t;
    GXvector N;

    scal = ProdScal(p->vitesse, s->N);
    if ( scal > 0.0 )
	return FALSE;

    t = ProdScal(s->N,Vector(p->position,s->A))/scal;
    if ( t < 0.0 || t > 1.0 )
	return FALSE;

    N.x = - p->vitesse.y;
    N.y = p->vitesse.x;

    t = ProdScal(N,Vector(p->position,s->A))/scal;
    if ( t < 0.0 || t > 1.0 )
	return FALSE;

    // Nouvelle position
    p->position.x += t * p->vitesse.x;
    p->position.y += t * p->vitesse.y;

    // Dessin de l'impact
    if ( imp )
	impact(p->position);

    // Nouveau vecteur vitesse
    t = 2 * scal / s->d;
    p->vitesse.x = p->vitesse.x - t * s->N.x;
    p->vitesse.y = p->vitesse.y - t * s->N.y;
    
    return TRUE;
}

/* Cherche une intersection entre le vecteur vitesse de p et c et affiche un
 * impact s'il y a et si imp = VRAI
 */
bool intersection_part_cer(Particule * p, Cercle * c, bool imp)
{
    double t, dCP, SQRr, d;
    GXpoint B; // position + vitesse de la particule
    GXpoint I;
    GXvector CI;
    bool out;

    B.x = p->position.x + p->vitesse.x;
    B.y = p->position.y + p->vitesse.y;

    SQRr = SQR(c->rayon);
    out = (SQR(c->C.x - B.x) + SQR(c->C.y - B.y)) > SQRr;

    t = ProjPntDte(p->position,B,c->C,&I);

    // le vecteur vitesse ne touche pas le cercle
    if ( out && (t < 0.0 || t > 1.0) )
	return FALSE;

    dCP = SQR(I.x - c->C.x) + SQR(I.y - c->C.y);

    // vecteur vitesse tangent ou ne passant pas par le cercle
    if ( dCP >= SQRr )
	return FALSE;

    // Nouvelle position
    d = sqrt((SQRr - dCP) / SqrNorm(p->vitesse));
    if ( out ) {
	p->position.x = I.x + d * (p->position.x - B.x);
	p->position.y = I.y + d * (p->position.y - B.y);
    } else {
	p->position.x = I.x + d * (p->position.x - B.x);
	p->position.y = I.y + d * (p->position.y - B.y);
    }

    // Dessin de l'impact
    if ( imp )
	impact(p->position);

    CI.x = p->position.x - c->C.x;
    CI.y = p->position.y - c->C.y;
    
    // Nouveau vecteur vitesse
    t = 2 * ProdScal(CI,p->vitesse) / SQR(c->rayon);
    p->vitesse.x = p->vitesse.x - t * CI.x;
    p->vitesse.y = p->vitesse.y - t * CI.y;

    return TRUE;
}

/* Cherche une collision entre p et tous les cercles
 * affiche un impact si imp = VRAI
 */
bool collision_cercles(Particule * p, bool imp)
{ 
    Cercle * tmp = cercles;
    while ( tmp != NULL ) {
	if ( intersection_part_cer(p,tmp,imp) )
	    return TRUE;
	tmp = tmp->suiv;
    }

    return FALSE;
}

/* Cherche une collision entre p et tous les segments
 * et affiche les impacts si imp = VRAI
 */
bool collision_segments(Particule * p, bool imp)
{
    Segment * tmp = segments;
    while ( tmp != NULL ) {
	if ( intersection_part_seg(p,tmp,imp) )
	    return TRUE;
	tmp = tmp->suiv;
    }
    return FALSE;
}

/* b poursuit a
 */
void poursuite(Particule *b, Particule *a)
{
    double norm,t;
    GXvector Pba;
    
    // Construction du vecteur PbPa
    Pba.x = (a->position.x + a->vitesse.x) - (b->position.x + b->vitesse.x);
    Pba.y = (a->position.y + a->vitesse.y) - (b->position.y + b->vitesse.y);
    norm = GXnorm(Pba);

    if ( ! IsZero(norm) ) {
        t = a->rappel * sqrt(SqrNorm(a->vitesse)) / norm;
        b->vitesse.x = ( 1 - a->rappel ) * b->vitesse.x + t * Pba.x;
        b->vitesse.y = ( 1 - a->rappel ) * b->vitesse.y + t * Pba.y;
    }
}

/* Fait avancer la particule p et la fait poursuivre une cible si cible != NULL
 */
void avance_particule(Particule * p, Particule *cible)
{
    p->position.x += p->vitesse.x;
    p->position.y += p->vitesse.y;
    
    if ( p->prec != NULL )
	poursuite(p,p->prec);
    if ( cible != NULL )
	poursuite(p,cible);
}

/* routine de déplacement d'un serpent, déplace toutes ses particules et
 * organise les poursuites
 */
void mouvoir_serpent(Serpent * tmp, bool chasseur)
{
    Particule * part = tmp->tete;
    
    if ( ! ( collision_segments(tmp->pilote, FALSE) 
		|| collision_cercles(tmp->pilote, FALSE) ) ) {
	if ( chasseur )
	    avance_particule(tmp->pilote,cible->tete);	
	else
	    avance_particule(tmp->pilote,NULL);
    }

    while ( part != NULL ) {
	if ( ! ( collision_segments(part,TRUE)
		    || collision_cercles(part,TRUE) ) )
	    avance_particule(part,NULL);	
	part = part->suiv;
    }
}

/* Déplacement de tous les serpents
 */
void mouvements(void)
{
    Serpent * tmp = serpents;

    while ( tmp != NULL ) {
	mouvoir_serpent(tmp,(tmp == cible) ? FALSE : TRUE);
	tmp = tmp->suiv;
    }
}
