#include "../include/serpent.h"

extern Segment * segments;
extern Cercle * cercles;
extern Serpent * serpents;
extern Serpent * cible;

/* Fait s'viter les particules a et b 
 */
void evite(Particule * a, Particule * b)
{
    GXvector ab, tmp;
    ab = gx_Vector(a->position,b->position);
    double norm = GXnorm(ab);
    bool croise;

    croise = InterSegment(a->position,
	    Vector_XY(a->position.x + a->vitesse.x, 
		a->position.y + a->vitesse.y),
	    b->position,
	    Vector_XY(b->position.x + b->vitesse.x, 
		b->position.y + b->vitesse.y),
	    NULL);
    
    if ( croise || (norm < SQR(a->rayon + b->rayon)) ) {
	if ( croise ) {
	    // change des vitesses
	    tmp = a->vitesse;
	    a->vitesse = b->vitesse;
	    b->vitesse = tmp;
	}
	// Nouvelle vitesse de a (pilote)
	a->vitesse.x = a->vitesse.x - ab.x;
	a->vitesse.y = a->vitesse.y - ab.y;
	// Nouvelle vitesse de b
	b->vitesse.x = b->vitesse.x + ab.x;
	b->vitesse.y = b->vitesse.y + ab.y;

	// Nouveaux tests de collision
	collision_segments(a,FALSE);
	collision_cercles(a,FALSE);
	collision_segments(b,FALSE);
	collision_cercles(b,FALSE);
    }
}

/* Fait s'viter les serpents s1 et s2
 */
void evitement(Serpent *s1, Serpent *s2)
{
    Particule * p1 = s1->pilote;
    Particule * p2 = s2->pilote;

    // Dtections entre le pilote de s2 et toutes les particules de s1
    while ( p1 != NULL ) {
	evite(s2->pilote,p1);
	p1 = p1->suiv;
    }

    // Dtections entre le pilote de s1 et toutes les particules de s2
    while ( p2 != NULL ) {
	evite(s1->pilote,p2);
	p2 = p2->suiv;
    }
}

/* Dtruit a si le vecteur vitesse de tete est sur le vecteur vitesse de a
 */
bool destruction(Particule *a, Particule *tete)
{
    bool croise;
    
    croise = InterSegment(tete->position,
	    Vector_XY(tete->position.x + tete->vitesse.x,
		tete->position.y + tete->vitesse.y),
	    a->position,a->prec->position,NULL);

    // S'il y a intersection, A est dtruite
    if ( croise ) {
	if ( a->prec != NULL && a->suiv != NULL )
	    poursuite(a->suiv,a->prec);
	if ( a == cible->tete ) {
	    cible->tete = cible->tete->suiv;
	    cible->pilote->suiv = cible->tete;
	    if ( cible->tete != NULL )
		cible->tete->prec = cible->pilote;
	}
	detruire_particule(a);

	return TRUE;
    }

    return FALSE;
}

/* Cherche les destructions possibles pour le serpent s
 */
void destructions(Serpent *s)
{
    Particule * tmp;
    tmp = cible->tete;

    // Cherche les destructions possibles du serpent s avec la cible
    while ( tmp != NULL ) {
	if ( destruction(tmp,s->tete) )
	    return;
	tmp = tmp->suiv;
    }
}

/* Destructions / vitements */
void rencontres(void)
{
    uchar tsl[3], rvb[3];

    // On ne traite pas les rencontres s'il n'y a qu'un seul serpent ou aucun
    // serpent
    if ( serpents == NULL || serpents->suiv == NULL )
	return;

    Serpent * tmp1 = serpents->suiv, * tmp2 = serpents;

    // vitements des chasseurs
    while ( tmp1 != NULL ) {
	if ( tmp1 != cible )
	    while ( tmp2 != NULL ) {
		if ( tmp2 != cible && tmp1 != tmp2 )
		    evitement(tmp1,tmp2);
		tmp2 = tmp2->suiv;
	    }
	tmp1 = tmp1->suiv;
	tmp2 = serpents;
    }
    
    // Destructions, uniquement s'il y a des chasseurs
    if ( cible->prec != NULL || cible->suiv != NULL )
	while ( tmp2 != NULL ) {
	    if ( tmp2 != cible ) {
		destructions(tmp2);
		// Cible tue ?
		if ( cible->tete == NULL ) {
		    detruire_serpent(cible);
		    cible = tmp2;
		    // Changement de couleur de cible
		    tsl[1] = 240;
		    tsl[2] = 120;
		    tsl[0] = aleatoire(20,220); // diffrente du rouge
		    tsl_rvb(tsl,rvb);
		    memcpy(cible->tete->col,rvb,3);
		}
	    }
	    tmp2 = tmp2->suiv;
	}
}
